# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


DependencyDetection.defer do
  named :httpclient

  HTTPCLIENT_MIN_VERSION = '2.2.0'

  depends_on do
    defined?(HTTPClient) && defined?(HTTPClient::VERSION)
  end

  depends_on do
    minimum_supported_version = Mmtrix::VersionNumber.new(HTTPCLIENT_MIN_VERSION)
    current_version = Mmtrix::VersionNumber.new(HTTPClient::VERSION)

    current_version >= minimum_supported_version
  end

  executes do
    ::Mmtrix::Agent.logger.info 'Installing HTTPClient instrumentation'
    require 'mmtrix/agent/cross_app_tracing'
    require 'mmtrix/agent/http_clients/httpclient_wrappers'
  end

  executes do
    class HTTPClient
      def do_get_block_with_mmtrix(req, proxy, conn, &block)
        wrapped_request = Mmtrix::Agent::HTTPClients::HTTPClientRequest.new(req)

        response = nil
        ::Mmtrix::Agent::CrossAppTracing.tl_trace_http_request(wrapped_request) do
          do_get_block_without_mmtrix(req, proxy, conn, &block)
          response = conn.pop
          conn.push response
          ::Mmtrix::Agent::HTTPClients::HTTPClientResponse.new(response)
        end
        response
      end

      alias do_get_block_without_mmtrix do_get_block
      alias do_get_block do_get_block_with_mmtrix
    end
  end
end
