# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


DependencyDetection.defer do
  @name = :rails2_error

  depends_on do
    defined?(ActionController) && defined?(ActionController::Base)
  end

  depends_on do
    defined?(::Rails) && ::Rails::VERSION::MAJOR.to_i == 2
  end

  executes do
    ::Mmtrix::Agent.logger.info 'Installing Rails 2 Error instrumentation'
  end

  executes do

    ActionController::Base.class_eval do

      # Make a note of an exception associated with the currently executing
      # controller action.  Note that this used to be available on Object
      # but we replaced that global method with Mmtrix::Agent#notice_error.
      # Use that one instead.
      #
      # @api public
      # @deprecated
      def mmtrix_notice_error(exception, custom_params = {})
        Mmtrix::Agent::Deprecator.deprecate("ActionController#mmtrix_notice_error",
                                              "Mmtrix::Agent#notice_error")

        Mmtrix::Agent::Transaction.notice_error exception, :custom_params => custom_params
      end

      def rescue_action_with_mmtrix_trace(exception)
        rescue_action_without_mmtrix_trace exception
        Mmtrix::Agent::Transaction.notice_error exception
      end

      # Compare with #alias_method_chain, which is not available in
      # Rails 1.1:
      alias_method :rescue_action_without_mmtrix_trace, :rescue_action
      alias_method :rescue_action, :rescue_action_with_mmtrix_trace
      protected :rescue_action

    end
  end
end
