# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require 'mmtrix/agent/instrumentation/middleware_proxy'

DependencyDetection.defer do
  named :rails_middleware

  depends_on do
    defined?(::Rails) && ::Rails::VERSION::MAJOR.to_i >= 3
  end

  depends_on do
    !::Mmtrix::Agent.config[:disable_middleware_instrumentation]
  end

  executes do
    ::Mmtrix::Agent.logger.info("Installing Rails 3+ middleware instrumentation")
    module ActionDispatch
      class MiddlewareStack
        class Middleware
          def build_with_mmtrix(app)
            # MiddlewareProxy.wrap guards against double-wrapping here.
            # We need to instrument the innermost app (usually a RouteSet),
            # which will never itself be the return value from #build, but will
            # instead be the initial value of the app argument.
            wrapped_app = ::Mmtrix::Agent::Instrumentation::MiddlewareProxy.wrap(app)
            result = build_without_mmtrix(wrapped_app)
            ::Mmtrix::Agent::Instrumentation::MiddlewareProxy.wrap(result)
          end

          alias_method :build_without_mmtrix, :build
          alias_method :build, :build_with_mmtrix
        end
      end
    end
  end
end
