# -*- ruby -*-
# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


DependencyDetection.defer do
  @name = :sequel

  depends_on do
    defined?(::Sequel)
  end

  depends_on do
    !Mmtrix::Agent.config[:disable_sequel_instrumentation] &&
    !Mmtrix::Agent.config[:disable_database_instrumentation]
  end

  def supported_sequel_version?
    Sequel.const_defined?( :MAJOR ) &&
      ( Sequel::MAJOR > 3 ||
        Sequel::MAJOR == 3 && Sequel::MINOR >= 37 )
  end

  executes do
    if supported_sequel_version?

      ::Mmtrix::Agent.logger.info 'Installing Sequel instrumentation'

      if Sequel::Database.respond_to?( :extension )
        Sequel::Database.extension :mmtrix_instrumentation
      else
        Mmtrix::Agent.logger.info "Detected Sequel version %s." % [ Sequel::VERSION ]
        Mmtrix::Agent.logger.info "Please see additional documentation: " +
          "https://mmtrix.com/docs/ruby/sequel-instrumentation"
      end

      Sequel.synchronize{Sequel::DATABASES.dup}.each do |db|
        db.extension :mmtrix_instrumentation
      end

      Sequel::Model.plugin :mmtrix_instrumentation

    else

      Mmtrix::Agent.logger.info "Sequel instrumentation requires at least version 3.37.0."

    end

  end

end
