# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


DependencyDetection.defer do
  named :typhoeus

  depends_on do
    defined?(Typhoeus) && defined?(Typhoeus::VERSION)
  end

  depends_on do
    Mmtrix::Agent::Instrumentation::TyphoeusTracing.is_supported_version?
  end

  executes do
    ::Mmtrix::Agent.logger.info 'Installing Typhoeus instrumentation'
    require 'mmtrix/agent/cross_app_tracing'
    require 'mmtrix/agent/http_clients/typhoeus_wrappers'
  end

  # Basic request tracing
  executes do
    Typhoeus.before do |request|
      Mmtrix::Agent::Instrumentation::TyphoeusTracing.trace(request)

      # Ensure that we always return a truthy value from the before block,
      # otherwise Typhoeus will bail out of the instrumentation.
      true
    end
  end

  # Apply single TT node for Hydra requests until async support
  executes do
    class Typhoeus::Hydra
      include Mmtrix::Agent::MethodTracer

      def run_with_mmtrix(*args)
        trace_execution_scoped("External/Multiple/Typhoeus::Hydra/run") do
          run_without_mmtrix(*args)
        end
      end

      alias run_without_mmtrix run
      alias run run_with_mmtrix
    end
  end
end


module Mmtrix::Agent::Instrumentation::TyphoeusTracing

  EARLIEST_VERSION = Mmtrix::VersionNumber.new("0.5.3")

  def self.is_supported_version?
    Mmtrix::VersionNumber.new(Typhoeus::VERSION) >= Mmtrix::Agent::Instrumentation::TyphoeusTracing::EARLIEST_VERSION
  end

  def self.request_is_hydra_enabled?(request)
    request.respond_to?(:hydra) && request.hydra
  end

  def self.trace(request)
    state = Mmtrix::Agent::TransactionState.tl_get
    if state.is_execution_traced? && !request_is_hydra_enabled?(request)
      wrapped_request = ::Mmtrix::Agent::HTTPClients::TyphoeusHTTPRequest.new(request)
      t0 = Time.now
      node = ::Mmtrix::Agent::CrossAppTracing.start_trace(state, t0, wrapped_request)
      callback = Proc.new do
        wrapped_response = ::Mmtrix::Agent::HTTPClients::TyphoeusHTTPResponse.new(request.response)
        ::Mmtrix::Agent::CrossAppTracing.finish_trace(state, t0, node, wrapped_request, wrapped_response)
      end
      request.on_complete.unshift(callback)
    end
  rescue => e
    Mmtrix::Agent.logger.error("Exception during trace setup for Typhoeus request", e)
  end
end
