# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require 'mmtrix/agent/mmtrix_service/marshaller'

module Mmtrix
  module Agent
    class MmtrixService
      # Marshal collector protocol with JSON when available
      class JsonMarshaller < Marshaller
        def initialize
          ::Mmtrix::Agent.logger.debug "Using JSON marshaller (#{Mmtrix::JSONWrapper.backend_name})"
          unless self.class.is_supported?
            ::Mmtrix::Agent.logger.warn "The JSON marshaller in use (#{Mmtrix::JSONWrapper.backend_name}) is not recommended. Ensure the 'json' gem is available in your application for better performance."
          end
          warn_for_yajl
        end

        OK_YAJL_VERSION = Mmtrix::VersionNumber.new("1.2.1")

        def warn_for_yajl
          if defined?(::Yajl)
            require 'yajl/version'
            if Mmtrix::VersionNumber.new(::Yajl::VERSION) < OK_YAJL_VERSION
              ::Mmtrix::Agent.logger.warn "Detected yajl-ruby version #{::Yajl::VERSION} which can cause segfaults with mmtrix_rpm's thread profiling features. We strongly recommend you upgrade to the latest yajl-ruby version available."
            end
          end
        rescue => err
          ::Mmtrix::Agent.logger.warn "Failed trying to watch for problematic yajl-ruby version.", err
        end

        def dump(ruby, opts={})
          prepared = prepare(ruby, opts)

          if opts[:skip_normalization]
            normalize_encodings = false
          else
            normalize_encodings = Agent.config[:normalize_json_string_encodings]
          end

          Mmtrix::JSONWrapper.dump(prepared, :normalize => normalize_encodings)
        end

        def load(data)
          if data.nil? || data.empty?
            ::Mmtrix::Agent.logger.error "Empty JSON response from collector: '#{data.inspect}'"
            return nil
          end

          return_value(Mmtrix::JSONWrapper.load(data))
        rescue => e
          ::Mmtrix::Agent.logger.debug "#{e.class.name} : #{e.message} encountered loading collector response: #{data}"
          raise
        end

        def default_encoder
          Encoders::Base64CompressedJSON
        end

        def format
          'json'
        end

        def self.is_supported?
          Mmtrix::JSONWrapper.usable_for_collector_serialization?
        end

        def self.human_readable?
          true # for some definitions of 'human'
        end
      end
    end
  end
end
