# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


module Mmtrix
  module Agent
    module Threading
      class AgentThread

        def self.create(label, &blk)
          ::Mmtrix::Agent.logger.debug("Creating Mmtrix thread: #{label}")
          wrapped_blk = Proc.new do
            begin
              blk.call
            rescue => e
              ::Mmtrix::Agent.logger.error("Thread #{label} exited with error", e)
            rescue Exception => e
              ::Mmtrix::Agent.logger.error("Thread #{label} exited with exception. Re-raising in case of interupt.", e)
              raise
            ensure
              ::Mmtrix::Agent.logger.debug("Exiting Mmtrix thread: #{label}")
            end
          end

          thread = backing_thread_class.new(&wrapped_blk)
          thread[:mmtrix_label] = label
          thread
        end

        # Simplifies testing if we don't directly use ::Thread.list, so keep
        # the accessor for it here on AgentThread to use and stub.
        def self.list
          backing_thread_class.list
        end

        def self.bucket_thread(thread, profile_agent_code) #THREAD_LOCAL_ACCESS
          if thread.key?(:mmtrix_label)
            profile_agent_code ? :agent : :ignore
          else
            state = TransactionState.tl_state_for(thread)
            if state.in_background_transaction?
              :background
            elsif state.in_web_transaction?
              :request
            else
              :other
            end
          end
        end

        def self.scrub_backtrace(thread, profile_agent_code)
          begin
            bt = thread.backtrace
          rescue Exception => e
            ::Mmtrix::Agent.logger.debug("Failed to backtrace #{thread.inspect}: #{e.class.name}: #{e.to_s}")
          end
          return nil unless bt
          bt.reject! { |t| t.include?('/mmtrix_rpm-') } unless profile_agent_code
          bt
        end

        # To allow tests to swap out Thread for a synchronous alternative,
        # surface the backing class we'll use from the class level.
        @backing_thread_class = ::Thread

        def self.backing_thread_class
          @backing_thread_class
        end

        def self.backing_thread_class=(clazz)
          @backing_thread_class = clazz
        end
      end
    end
  end
end
