# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require 'mmtrix/control/frameworks/rails'
require 'mmtrix/rack/error_collector'

module Mmtrix
  class Control
    module Frameworks
      # Control subclass instantiated when Rails is detected.  Contains
      # Rails 3.0+  specific configuration, instrumentation, environment values,
      # etc. Many methods are inherited from the
      # Mmtrix::Control::Frameworks::Rails class, where the two do
      # not differ
      class Rails3 < Mmtrix::Control::Frameworks::Rails

        def env
          @env ||= ::Rails.env.to_s
        end

        def rails_root
          ::Rails.root.to_s
        end

        def vendor_root
          @vendor_root ||= File.join(root,'vendor','rails')
        end

        def version
          @rails_version ||= Mmtrix::VersionNumber.new(::Rails::VERSION::STRING)
        end

        protected

        def install_shim
          super
          ActiveSupport.on_load(:action_controller) do
            include Mmtrix::Agent::Instrumentation::ControllerInstrumentation::Shim
          end
        end
      end
    end
  end
end
