# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


require 'capistrano/framework'

namespace :mmtrix do

  # notifies Mmtrix of a deployment
  desc "Record a deployment in Mmtrix (mmtrix.com)"
  task :notice_deployment do
    run_locally do
      environment = fetch(:mmtrix_rails_env, fetch(:rack_env, fetch(:rails_env, "production")))

      require 'mmtrix/cli/command.rb'

      begin
        # allow overrides to be defined for revision, description, changelog, appname, and user
        rev         = fetch(:mmtrix_revision)
        description = fetch(:mmtrix_desc)
        changelog   = fetch(:mmtrix_changelog)
        appname     = fetch(:mmtrix_appname)
        user        = fetch(:mmtrix_user)
        license_key = fetch(:mmtrix_license_key)

        unless scm == :none
          changelog ||= lookup_changelog
          rev       ||= fetch(:current_revision)
        end

        new_revision = rev
        deploy_options = {
          :environment => environment,
          :revision    => new_revision,
          :changelog   => changelog,
          :description => description,
          :appname     => appname,
          :user        => user,
          :license_key => license_key
        }

        debug "Uploading deployment to Mmtrix"
        deployment = Mmtrix::Cli::Deployments.new deploy_options
        deployment.run
        info "Uploaded deployment information to Mmtrix"

      rescue Mmtrix::Cli::Command::CommandFailure => e
        info e.message
      rescue => e
        info "Error creating Mmtrix deployment (#{e})\n#{e.backtrace.join("\n")}"
      end
    end
  end

  def lookup_changelog
    debug "Retrieving changelog for Mmtrix Deployment details"
    previous_revision = fetch(:previous_revision)
    current_revision = fetch(:current_revision)

    if scm == :git
      log_command = "git log --no-color --pretty=format:'  * %an: %s' " +
                    "--abbrev-commit --no-merges #{previous_revision}..#{current_revision}"
      `#{log_command}`
    end
  end
end
