# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


make_notify_task = Proc.new do

  namespace :mmtrix do

    # on all deployments, notify Mmtrix
    desc "Record a deployment in Mmtrix (mmtrix.com)"
    task :notice_deployment, :roles => :app, :except => {:no_release => true } do
      rails_env = fetch(:mmtrix_rails_env, fetch(:rails_env, "production"))

      require 'mmtrix/cli/command'

      begin
        # allow overrides to be defined for revision, description, changelog, appname, and user
        rev         = fetch(:mmtrix_revision)    if exists?(:mmtrix_revision)
        description = fetch(:mmtrix_desc)        if exists?(:mmtrix_desc)
        changelog   = fetch(:mmtrix_changelog)   if exists?(:mmtrix_changelog)
        appname     = fetch(:mmtrix_appname)     if exists?(:mmtrix_appname)
        user        = fetch(:mmtrix_user)        if exists?(:mmtrix_user)
        license_key = fetch(:mmtrix_license_key) if exists?(:mmtrix_license_key)

        unless scm == :none
          changelog = lookup_changelog(changelog)
          rev       = lookup_rev(rev)
        end

        new_revision = rev
        deploy_options = {
          :environment => rails_env,
          :revision    => new_revision,
          :changelog   => changelog,
          :description => description,
          :appname     => appname,
          :user        => user,
          :license_key => license_key
        }

        logger.debug "Uploading deployment to Mmtrix"
        deployment = Mmtrix::Cli::Deployments.new deploy_options
        deployment.run
        logger.info "Uploaded deployment information to Mmtrix"

      rescue Mmtrix::Cli::Command::CommandFailure => e
        logger.info e.message
      rescue Capistrano::CommandError
        logger.info "Unable to notify Mmtrix of the deployment... skipping"
      rescue => e
        logger.info "Error creating Mmtrix deployment (#{e})\n#{e.backtrace.join("\n")}"
      end
    end

    def lookup_changelog(changelog)
      if !changelog
        logger.debug "Getting log of changes for Mmtrix Deployment details"
        from_revision = source.next_revision(current_revision)

        if scm == :git
          log_command = "git log --no-color --pretty=format:'  * %an: %s' " +
            "--abbrev-commit --no-merges #{previous_revision}..#{real_revision}"
        else
          log_command = "#{source.log(from_revision)}"
        end

        changelog = `#{log_command}`
      end
      changelog
    end

    def lookup_rev(rev)
      if rev.nil?
        rev = source.query_revision(source.head()) do |cmd|
          logger.debug "executing locally: '#{cmd}'"
          `#{cmd}`
        end

        rev = rev[0..6] if scm == :git
      end
      rev
    end
  end
end

require 'capistrano/version'

if defined?(Capistrano::Version::MAJOR) && Capistrano::Version::MAJOR < 2
  STDERR.puts "Unable to load #{__FILE__}\nMmtrix Capistrano hooks require at least version 2.0.0"
else
  instance = Capistrano::Configuration.instance

  if instance
    instance.load(&make_notify_task)
  else
    make_notify_task.call
  end
end
