# encoding: utf-8
# This file is distributed under Mmtrix's license terms.


# == Mmtrix Initialization
#
# When installed as a gem, you can activate the Mmtrix agent one of the following ways:
#
# For Rails, add:
#    config.gem 'mmtrix_rpm'
# to your initialization sequence.
#
# For merb, do
#    dependency 'mmtrix_rpm'
# in the Merb config/init.rb
#
# For Sinatra, do
#    require 'mmtrix_rpm'
# after requiring 'sinatra'.
#
# For other frameworks, or to manage the agent manually, invoke Mmtrix::Agent#manual_start
# directly.
#

require 'mmtrix/control'
if defined?(Merb) && defined?(Merb::BootLoader)
  module Mmtrix
    class MerbBootLoader < Merb::BootLoader
      after Merb::BootLoader::ChooseAdapter
      def self.run
        Mmtrix::Control.instance.init_plugin
      end
    end
  end
elsif defined?(Rails::VERSION)
  if Rails::VERSION::MAJOR.to_i >= 3
    module Mmtrix
      class Railtie < Rails::Railtie

        initializer "mmtrix_rpm.start_plugin" do |app|
          Mmtrix::Control.instance.init_plugin(:config => app.config)
        end
      end
    end
  else
    # After version 2.0 of Rails we can access the configuration directly.
    # We need it to add dev mode routes after initialization finished.
    config = nil
    config = Rails.configuration if Rails.respond_to?(:configuration)
    Mmtrix::Control.instance.init_plugin :config => config
  end
else
  Mmtrix::Control.instance.init_plugin
end
